const { dialog } = require('electron').remote

let index = {
    // Handle the about button popup
    about: function() {
        let c = document.createElement("div");
        c.innerHTML = `<img src="iconsmol.png"> 
        <div>
            <b>Juicebox</b>
        </div>
        <br>
        <div>
            Version 0.1.1
        </div>
        <br>
        <div>
            For issues, bug reports, suggestions: 
        </div>
        <br>
        <div>
            Github: <a href="https://github.com/kelr">Kyrotobi</a> or <a href="https://github.com/evanito">Evanito</a>
        </div>
        <br>
        <div>
            Discord: kyrotobi#9714 
        </div>
        `;
        c.className = "about-content"
        asticode.modaler.setContent(c);
        asticode.modaler.show();

    },

    newRedemptionPrompt: function() {
        let c = document.createElement("div");
        c.className = "about-content"
        c.innerHTML = `
            <div id="login-error">
                Enter the name of the redemption. 
            </div>
            <div id="login-error">
                Make sure it matches the one on your channel exactly!
            </div>
            <div id="login-error">
                Note: Twitch's default redemptions don't work with Juicebox.
            </div>
            <br>
            <div id="redemption-error" style="display: none;">Invalid character in redemption. Please avoid using quotes, apostrophes or special characters.</div>
            <form id="new-redemption-form">
                <input type="text" id="redemption-name" name="redemption-name" placeholder="Juicebox"><br>
                <input type="submit" value="Create">
            </form>`;
        asticode.modaler.setContent(c);
        asticode.modaler.show();

        let form = document.getElementById('new-redemption-form');
        form.addEventListener('submit', (event) => {
            event.preventDefault();
            let redemptionErrMsg = document.getElementById('redemption-error');

            let value = form.elements['redemption-name'].value;
            if (value !== "") {
                if (value.includes("\"") || value.includes("\'") || value.includes("(") || value.includes(")") || value.includes("<") || value.includes(">")) {
                    redemptionErrMsg.style.display = "block"
                } else {
                    index.newRedemption("song", value, false)
                    redemptionErrMsg.style.display = "none"
                    asticode.modaler.hide();
                }
            }
        });
    },

    // Add a new redemption container. A container consists of a collapsable button, a delete button and the content.
    newRedemption: function(group, name, fromBackend) {
        console.log("newredemption", group, name, fromBackend)

        

        var container = document.createElement("div");
        container.setAttribute("id", name + "-container");

        var button = document.createElement("button");
        button.setAttribute("type", "button");
        button.setAttribute("class", "collapsible");
        button.innerHTML = name
        button.addEventListener("click", collapsableEventListener);
        container.appendChild(button)

        if (group === "song") {
            var delButton = document.createElement("i");
            delButton.setAttribute("style", "float: right");
            delButton.setAttribute("class", "fa fa-trash redemption-remove-btn");
            delButton.setAttribute("onclick", `index.removeRedemption('` + group + `', '` + name + `')`)
            container.appendChild(delButton)
        } else {
            // Jank dummy button to make the event handlers work
            var dummy = document.createElement("div");
            container.appendChild(dummy)
        }

        var content = document.createElement("div");
        content.setAttribute("class", "collapsiblecontent");
        content.innerHTML = `
            Add song  <i class="fa fa-plus redemption-new" onclick="index.newRedemptionContent('` + group + `', '` + name +`', false)"></i> 
            <table id="` + name + `-table" style="width:80%"></table>`
        container.appendChild(content)

        if (group === "song") {
            console.log(name + "-table")
            document.getElementById("channel-points").append(container)
        } else if (group === "mod") {
            document.getElementById("mod-actions").append(container)
        }

        // Tell the backend theres a new redemption
        if (!fromBackend){
            astilectron.sendMessage({
                "name": "redemptionadd",
                "payload": encodeURIComponent(group) + "," + encodeURIComponent(name)
            })
        }
    },

    // Remove a redemption container.
    removeRedemption: function(group, redemption) {
        // Delete all registered songs, if any.
        let table = document.getElementById(redemption + '-table');
        Array.from(table.rows).forEach((row) => {
            let index = row.rowIndex
            console.log("DELETE ROW " + index.toString())
            table.deleteRow(index)

            // Tell the backend to update the data struct and remove it from the file
            astilectron.sendMessage({
                "name": "filedel",
                "payload": encodeURIComponent(group) + "," + encodeURIComponent(redemption) + "," + encodeURIComponent(index.toString())
            })
        });

        // Delete the redemption itself
        var elem = document.getElementById(redemption + "-container");
        elem.parentNode.removeChild(elem);

        // Tell the backend to delete this redemption
        astilectron.sendMessage({
            "name": "redemptiondel",
            "payload": encodeURIComponent(group) + "," + encodeURIComponent(redemption)
        })
    },

    // Prompt the user to select song files to add and add them if they are not already added to this redemption.
    newRedemptionContent: function(group, redemption, fileName) {
        dialog.showOpenDialog({filters: [{name: 'Songs', extensions: ['mp3', 'flac', 'ogg', 'wav']}],
            properties: ['openFile', 'multiSelections'] 
        }).then(result => {
          if (!result.canceled) {
            result.filePaths.forEach((item) => {
                index.addSong(group, redemption, item, false)
            })
          }
        }).catch(err => {
          console.log(err)
        })
    },

    addSong: function(group, redemption, item, fromBackend) {
        console.log("addsong", redemption, item, fromBackend)
        let fileName = item.split("\\").slice(-1)[0]
        let table = document.getElementById(redemption + '-table');
        console.log(redemption + '-table')
        // Don't add if it already exists
        for (var i = 0, row; row = table.rows[i]; i++) {
            if (row.getAttribute("filename") === fileName) {
                console.log("already exists")
                return
            }
        }

        // Add a new row to the table
        var row = table.insertRow(-1);
        row.setAttribute("style", "text-align:left")
        row.setAttribute("filename", fileName)
        row.innerHTML = item + 
            `<i class="fa fa-trash song-remove-btn" style="float: right;" onclick="index.removeRedemptionContent('` + group + `', '` + redemption + `', '` + fileName + `')"></i>`

        if (!fromBackend) {
            // Update the collapsable buttons view to account for the new element
            let content = table.closest(".collapsiblecontent")
            content.style.maxHeight = content.scrollHeight + "px";

            // Tell the backend to update the data struct and the file
            astilectron.sendMessage({
                "name": "fileadd",
                "payload": encodeURIComponent(group) + "," + encodeURIComponent(redemption) + "," + encodeURIComponent(item)
            })
        }
    },

    // Delete a song file if it exists. Must do this by index, but must identify the song by pseudo id.
    removeRedemptionContent: function(group, redemption, songName) {
        let table = document.getElementById(redemption + '-table');
        Array.from(table.rows).forEach((row) => {
            if (row.getAttribute("filename") === songName) {
                let index = row.rowIndex
                table.deleteRow(index)
                
                // Update the collapsable buttons view to account for the new element
                let content = table.closest(".collapsiblecontent")
                content.style.maxHeight = content.scrollHeight + "px";

                // Tell the backend to update the data struct and remove it from the file
                astilectron.sendMessage({
                    "name": "filedel",
                    "payload": encodeURIComponent(group) + "," + encodeURIComponent(redemption) + "," + encodeURIComponent(index.toString())
                })
                // TODO: this doesnt actually break the loop early, use a real for loop.
                return
            }
        });
    },

    // Switch to the home view
    viewHome: function() {
        var jason = document.getElementById("jasonedit");
        var logconsole = document.getElementById("logconsole")
        if (jason.style.display === "block") {
           jason.style.display = "none";
           logconsole.style.display = "block";
        }
    },

    // Switch to the edit view
    viewEdit: function() {
        var jason = document.getElementById("jasonedit");
        var logconsole = document.getElementById("logconsole")
        if (jason.style.display === "none") {
           jason.style.display = "block";
           logconsole.style.display = "none";
        }
    },

    // Write a line to the logger view
    log: function(msg) {
        var node = document.createTextNode(msg.logmessage);
        var br = document.createElement("br");
        var elem = document.getElementById("logconsole")

        elem.appendChild(node);
        elem.appendChild(br);
        elem.scrollTop = elem.scrollHeight;
    },

    // Show the login view
    auth: function() {
        let c = document.createElement("div");
        c.className = "about-content"
        c.innerHTML = `
            <button id="connect-with-twitch" class="login-button" onclick="linkOpen('https://juicebox.dev/login')">
                <img src="twitchicon.png">
                <span class="login-text">Login with Twitch</span>
            </button>
            <div id="login-error" style="display: none;">Error parsing login info! Please ensure the info was copied correctly from the site.</div>
            <form id="tokenform">
                <input type="text" id="token" name="token" placeholder="Paste info here"><br>
                <input type="submit" value="Authenticate">
            </form>`;
        asticode.modaler.setContent(c);
        asticode.modaler.show();

        let form = document.getElementById('tokenform');
        form.addEventListener('submit', (event) => {
            event.preventDefault();
            let loginErrMsg = document.getElementById('login-error');

            let value = form.elements['token'].value;
            let splitToken = value.split(';');
            if (splitToken.length === 4) {
                // Send token and ID. In the future the client id and username may be used.
                let token = splitToken[0].split('=')[1]
                let userName = splitToken[1].split('=')[1]
                let userID = splitToken[2].split('=')[1]
                astilectron.sendMessage({
                    "name": "auth",
                    "payload": encodeURIComponent(token) + "," + encodeURIComponent(userID) + "," + encodeURIComponent(userName),
                })
                loginErrMsg.style.display = "none"
                asticode.modaler.hide();
            } else {
                loginErrMsg.style.display = "block"
            }
        });

    },

    init: function() {
        asticode.modaler.init();

        // Wait for astilectron to be ready
        document.addEventListener('astilectron-ready', function() {
            // Listen for messages from the backend
            index.listen();
        })

        // Force all clicks on <a> link tags to use an external browser
        document.addEventListener('click', event => {
          if (event.target.tagName.toLowerCase() === 'a') {
            event.preventDefault();
            require("electron").shell.openExternal(event.target.href);
          }
        });

        // Attach event listeners to all collapsible buttons
        var coll = document.getElementsByClassName("collapsible");
        var i;
        for (i = 0; i < coll.length; i++) {
          coll[i].addEventListener("click", collapsableEventListener);
        }
    },

    // Listen to messages from the backend
    listen: function() {
        astilectron.onMessage(function(message) {
            switch (message.name) {
                case "about":
                    index.about();
                    break;
                case "log":
                    index.log(message.payload);
                    break;
                case "auth":
                    index.auth();
                    break;
                case "viewEdit": 
                    index.viewEdit();
                    break;
                case "viewHome": 
                    index.viewHome();
                    break;
                case "newRedemption":
                    let split = message.payload.split(",")
                    index.newRedemption(split[0], split[1], true);
                    break;
                case "addSong":
                    let songSplit = message.payload.split(",")
                    index.addSong(songSplit[0], songSplit[1], songSplit[2], true)
            }
        });
    }
};


// Open a link in an external browser
function linkOpen(url) {
    require('electron').shell.openExternal(url);
}

// Event listener that collapses or expands the elemnts next-next sibling
// It has to be next-next since for the button the next sibling is the trash can
function collapsableEventListener() {
    this.classList.toggle("active");
    var content = this.nextElementSibling.nextElementSibling;
    if (content.style.maxHeight){
      content.style.maxHeight = null;
    } else {
      content.style.maxHeight = content.scrollHeight + "px";
    }
}
